
#include "Simulation.h"

//////////////////////////////////////////////////////
//  KONSTANTY
//////////////////////////////////////////////////////

const int  POCET_FRONT   =  4; // pocet udu profesora
const int  POCET_MIST    = 10; // komaru na jednom udu
const TIME DOBA_OBSLUHY  =  1; // casovych jednotek na napiti komara
const TIME DOBA_INKUBACE = 20; // za jak dlouho novy komar
const int  POCET_POTOMKU = 10; // kolik novych potomku

typedef class CFronta CFronta;
typedef class CKomar CKomar;
typedef class CMisto CMisto;
typedef CFronta *CFrontaPtr;
typedef CKomar *CKomarPtr;
typedef CMisto *CMistoPtr;

//////////////////////////////////////////////////////
//  FRONTY - NA KAZDE RUCE/NOZE
//////////////////////////////////////////////////////

class CFronta : public CHead {
public:
  CFronta();
  ~CFronta();
  void ActivateAll(TIME time);
  int cislo;
  int volnych;
private:
  static int pocet;
  CMistoPtr mista[POCET_MIST];
};

//////////////////////////////////////////////////////
//  CHOVANI JEDNOHO KOMARA
//////////////////////////////////////////////////////

class CKomar : public CProcess {
public:
  void Run();
  int cislo;
  TIME zarazen;
private:
  static int pocet;
};

//////////////////////////////////////////////////////
//  CHOVANI JEDNOHO MISTA NA RUCE/NOZE
//////////////////////////////////////////////////////

class CMisto : public CProcess {
public:
  CMisto(CFrontaPtr fronta, int cislo);
  void Run();
  int cislo;
private:
  CFrontaPtr F;
};

//////////////////////////////////////////////////////
//  FRONTY - NA KAZDE RUCE/NOZE
//////////////////////////////////////////////////////

CFronta::CFronta() {
  cislo = ++pocet;
  for (int i=0; i<POCET_MIST; i++) mista[i] = new CMisto(this,i+1);
  volnych = POCET_MIST;
}

CFronta::~CFronta() {
  for (int i=0; i<POCET_MIST; i++) delete mista[i];
}

void CFronta::ActivateAll(TIME time) {
  for (int i=0; i<POCET_MIST; i++) mista[i]->ActivateAt(time);
}

int CFronta::pocet = 0;

CFrontaPtr Fronta[POCET_FRONT];

//////////////////////////////////////////////////////
//  CHOVANI JEDNOHO KOMARA
//////////////////////////////////////////////////////

void CKomar::Run() {
  cislo = ++pocet;

  int i;
  CFrontaPtr nejkratsi = Fronta[0];
  for (i=0; i<POCET_FRONT; i++) {
    if (Fronta[i]->Cardinal()<nejkratsi->Cardinal()) {
      nejkratsi = Fronta[i];
    }
    else if (Fronta[i]->Cardinal()==nejkratsi->Cardinal()) {
      if (Fronta[i]->volnych>nejkratsi->volnych) {
        nejkratsi = Fronta[i];
      }
    }
  }

  Into(nejkratsi);

  zarazen = Time();

  printf("%3.0f: %d.komar se narodil a zaradil do %d.fronty jako %d. v rade\n",Time(),cislo,nejkratsi->cislo,nejkratsi->Cardinal());

  nejkratsi->ActivateAll(Time());

  Passivate();

  for (i=0; i<POCET_POTOMKU; i++) {
   (new CKomar)->ActivateAt(Time()+DOBA_INKUBACE);
  }
  
  printf("%3.0f: %d.komar se rozmnozil a umrel\n",Time(),cislo);
}

int CKomar::pocet = 0;

//////////////////////////////////////////////////////
//  CHOVANI JEDNOHO MISTA NA RUCE/NOZE
//////////////////////////////////////////////////////

CMisto::CMisto(CFrontaPtr fronta, int cislo) {
  F = fronta;
  this->cislo = cislo;
}

void CMisto::Run() {
  while (true) {
    while (F->Empty()) Passivate();
    F->volnych--;
    CKomarPtr K = (CKomarPtr)F->First();
    K->Out();
    printf("%3.0f: %d.komar cekal %.0f casovych jednotek\n",Time(),K->cislo,Time()-K->zarazen);
    printf("%3.0f: %d.komar prileta na misto %d na udu c.%d\n",Time(),K->cislo,cislo,F->cislo);
    Hold(DOBA_OBSLUHY);
    printf("%3.0f: %d.komar odleta z mista %d na udu c.%d\n",Time(),K->cislo,cislo,F->cislo);
    K->ActivateAt(Time());
    F->volnych++;
  }
}

//////////////////////////////////////////////////////
//  HLAVNI PROCES
//////////////////////////////////////////////////////

void CSimulation::Run() {
 printf("=== Starting simulation === \n");

 // PROMENNE
 int i;

 // VYTVOR FRONTY
 for (i=0; i<POCET_FRONT; i++) { Fronta[i] = new CFronta; }

 // VYTVOR PRVNIHO KOMARA
 (new CKomar)->ActivateAt(Time());
 
 // DOBA TRVANI SIMULACE
 Hold(70);

 // ZNIC FRONTY
 for (i=0; i<POCET_FRONT; i++) delete Fronta[i];

 printf("=== Stopping simulation === \n");
}
