library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity MH7472 is
port (
  J:  in  STD_LOGIC_VECTOR(0 to 2);
  K:  in  STD_LOGIC_VECTOR(0 to 2);
  R:  in  STD_LOGIC;
  S:  in  STD_LOGIC;
  CL: in  STD_LOGIC;
  Q:  out STD_LOGIC;
  Qn: out STD_LOGIC
);
end MH7472;

architecture VALEKO of MH7472 is
signal NAND1,NAND2,AND1,AND2,AND3,AND4,NOR1,NOR2,AND5,AND6 : STD_LOGIC;
signal INT1,INT2,INT3,INT4,INT5,INT6,INT7,INT8,INT9,INT0  : STD_LOGIC;
signal INTQ,INTQn : STD_LOGIC;
begin

NAND1 <= not(INT1 and INTQn and S);
INTQ <= 'X',NAND1 after 15 ns;
Q <= INTQ;

NAND2 <= not(INTQ and INT2 and R);
INTQn <= 'X',NAND2 after 15 ns;
Qn <= INTQn;

AND1 <= (INT2 and R and INT4);
INT3 <= 'X',AND1 after 10 ns;

AND2 <= (S and INT1 and INT6);
INT5 <= 'X',AND2 after 10 ns;

AND3 <= (INT4 and R);
INT7 <= 'X',AND3 after 10 ns;

AND4 <= (INT6 and S);
INT8 <= 'X',AND4 after 10 ns;

NOR1 <= (INT7 nor INT9);
INT6 <= 'X',NOR1 after 10 ns;

NOR2 <= (INT8 nor INT0);
INT4 <= 'X',NOR2 after 10 ns;

AND5 <= (R and INTQn and J(0) and J(1) and J(2) and CL);
INT9 <= 'X',AND5 after 10 ns;

AND6 <= (INTQ and S and CL and K(0) and K(1) and K(2));
INT0 <= 'X',AND6 after 10 ns;

INT1 <= (not INT3) or CL;
INT2 <= (not INT5) or CL;

end VALEKO;

---------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity SHIFTER is generic(N : integer := 8);
port (
  DIN   : in  STD_LOGIC;
  DATA  : out STD_LOGIC_VECTOR(0 to N-1);
  CLK   : in  STD_LOGIC;
  RESET : in  STD_LOGIC
);
end SHIFTER;

architecture VALEKO of SHIFTER is

  signal INTERNAL_J : STD_LOGIC_VECTOR(0 to N);
  signal INTERNAL_K : STD_LOGIC_VECTOR(0 to N);

  component MH7472
  port (
    J:  in  STD_LOGIC_VECTOR(0 to 2);
    K:  in  STD_LOGIC_VECTOR(0 to 2);
    R:  in  STD_LOGIC;
    S:  in  STD_LOGIC;
    CL: in  STD_LOGIC;
    Q:  out STD_LOGIC;
    Qn: out STD_LOGIC
  );
  end component;

begin

  INTERNAL_J(0) <= DIN;
  INTERNAL_K(0) <= NOT(DIN);

  SHIFTER: for i in 0 to N-1 generate
    JK: MH7472 port map (
     J(0) => INTERNAL_J(i),
     J(1) => '1',
     J(2) => '1',
     K(0) => INTERNAL_K(i),
     K(1) => '1',
     K(2) => '1',
     R    => RESET,
     S    => '1',
     CL   => CLK,
     Q    => INTERNAL_J(i+1),
     Qn   => INTERNAL_K(i+1)
    );
  end generate;

  OUTPUT: for i in 0 to N-1 generate
    DATA(i) <= INTERNAL_J(i+1);
  end generate;

end VALEKO;

---------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
entity TESTBENCH is
end TESTBENCH;

architecture TB_SHIFTER of TESTBENCH is

  constant N : integer := 8;  
  constant delay : time := 200 ns;

  signal DIN   : STD_LOGIC;
  signal DATA  : STD_LOGIC_VECTOR(0 to N-1);
  signal CLK   : STD_LOGIC;
  signal RESET : STD_LOGIC;

  component SHIFTER
  generic(N : integer);
  port (
    DIN   : in  STD_LOGIC;
    DATA  : out STD_LOGIC_VECTOR(0 to N-1);
    CLK   : in  STD_LOGIC;
    RESET : in  STD_LOGIC
  );
  end component;

begin  

  TEST: SHIFTER generic map(N) port map (DIN,DATA,CLK,RESET);

  process
  begin
    DIN<='0'; CLK<='0';
    RESET<='1'; RESET<='0'; wait for 200 ns; RESET<='1'; wait for 10 ns;
    DIN<='1'; CLK<='1'; wait for delay; CLK<='0'; wait for delay;
    DIN<='0'; CLK<='1'; wait for delay; CLK<='0'; wait for delay;
    DIN<='0'; CLK<='1'; wait for delay; CLK<='0'; wait for delay;
    DIN<='1'; CLK<='1'; wait for delay; CLK<='0'; wait for delay;
    DIN<='0'; CLK<='1'; wait for delay; CLK<='0'; wait for delay;
    DIN<='0'; CLK<='1'; wait for delay; CLK<='0'; wait for delay;
    DIN<='0'; CLK<='1'; wait for delay; CLK<='0'; wait for delay;
    DIN<='1'; CLK<='1'; wait for delay; CLK<='0'; wait for delay;
    DIN<='0'; CLK<='1'; wait for delay; CLK<='0'; wait for delay;
    DIN<='0'; CLK<='1'; wait for delay; CLK<='0'; wait for delay;
    DIN<='1'; CLK<='1'; wait for delay; CLK<='0'; wait for delay;
    DIN<='0'; CLK<='1'; wait for delay; CLK<='0'; wait for delay;
  end process;

end TB_SHIFTER;
